import json
from hashlib import sha1

from apispec.utils import deepupdate


class Schema:
    def __init__(self, id, name, properties):
        self._id = id
        self._name = name
        self._properties = properties

    def id(self):
        return self._id

    def to_ref(self, schema=False):
        return {
            'description': self._name,
            'content': {'application/json': {'schema':
                        schema and self.to_schema() or self.id()}}}

    def to_schema(self):
        return dict(properties=self._properties)


class ModelSchema(Schema):
    def __init__(self, model, fields, properties=None):
        self.model = model
        self.fields = fields
        self._properties = properties

    def id(self):
        hash = sha1(json.dumps(self.to_schema()).encode()).hexdigest()[:10]
        return f"{self.model.__name__}.{hash}"

    def to_ref(self, schema=False):
        return {
            'description': str(self.model._meta.verbose_name),
            'content': {'application/json': {'schema':
                        schema and self.to_schema() or self.id()}}}

    def default_json_repr_schema(self):
        from pfx.pfxcore.views.fields import FieldType
        return dict(
            pk=dict(
                type=FieldType.to_apidoc(FieldType.from_model_field(
                    self.model._meta.pk.__class__)),
                readonly=True),
            resource_name=dict(type='string', readonly=True))

    def to_schema(self):
        properties = getattr(
            self.model, 'json_repr_schema', self.default_json_repr_schema)()
        for name, field in self.fields.items():
            doc = (
                hasattr(self.model, 'apidoc') and
                self.model.apidoc.get(field.name) or {})
            properties[name] = deepupdate(field.to_apidoc(), doc.copy())
        if self._properties:
            properties.update(self._properties)
        return dict(properties=properties)


class ModelListSchema(ModelSchema):
    def to_ref(self):
        return {
            'description': str(self.model._meta.verbose_name_plural),
            'content': {'application/json': {'schema': self.id()}}}

    def to_schema(self):
        model = super().to_schema()
        return dict(properties=dict(
            items=dict(
                type='object', format=str(self.model._meta.verbose_name),
                **model),
            meta=dict(type='object')))
